/********************************************************************************
 *  Net.cpp                                                                     *
 *                                                                              *
 *                                                                              *
 *  Petri Nets simulation program                                               *
 *  is written in C++  and it is a part of the PhD research of                  *
 *  Polina Reshetova performed in 2010-2014 at                                  *
 *  Biosystems data analysis group, SILS,                                       *
 *  University van Amsterdam                                                    *
 *                                                                              *
 *  Written by Polina Reshetova                                                 *
 *                                                                              *
 *                                                                              *
 *  This is free software: you can redistribute it and/or modify                *
 *  it under the terms of the GNU Lesser General Public License as published by *
 *  the Free Software Foundation, either version 3 of the License, or           *
 *  (at your option) any later version.                                         *
 *                                                                              *
 *  the software is distributed in the hope that it will be useful,             *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *  GNU Lesser General Public License for more details.                         *
 *                                                                              *
 *  You should have received a copy of the GNU Lesser General Public License    *
 *  along with this software.  If not, see <http://www.gnu.org/licenses/>.      *
 ********************************************************************************/

#include "Net.h"

Net::Net() {
	places = new vector<Place*>;
	tblocks = new vector<TBlock*>;
}

Net::~Net() {
	// TODO Auto-generated destructor stub
}

//in transition blocks all transitions have to have one pre-place.
//if it is not true the function will not work correctly
vector<TBlock*> Net::getEnabledTBlocks(){
	vector<TBlock*> enabled;

	for (vector<TBlock*>::const_iterator i = tblocks->begin(); i!= tblocks->end(); i++)
	{
		TBlock* tb = *i;

		if (tb->isEnabled())
		{
			enabled.push_back(tb);
		}
	}

	return enabled;
}

vector<Place*> *Net::getPlaces(){
	return places;
}

vector<Transition*> *Net::getTransitions(){
	vector<Transition*> *transitions = new vector<Transition*>();

	for (vector<TBlock*>::const_iterator i = tblocks->begin(); i!= tblocks->end(); i++)
		{
		TBlock* tb = *i;
		vector<Transition*> *ts = tb->getTransitions();
		for (vector<Transition*>::const_iterator iter = ts->begin(); iter != ts->end(); iter++)
		{
			Transition* t = *iter;
			transitions->push_back(t);
		}
	}

	return transitions;
}


Place *Net::findPlaceByName(const std::string& name)
{
	for (vector<Place*>::const_iterator iter = places->begin(); iter != places->end(); iter++)
	{
		Place* p = *iter;
		if ((p->getName()) == name)
		{
			return p;
		}
	}
	return NULL;
}

TBlock *Net::findTBlockByName(const std::string& name)
{
	for (vector<TBlock*>::const_iterator iter = tblocks->begin(); iter != tblocks->end(); iter++)
	{
		TBlock* t = *iter;
		if ((t->getName()) == name)
		{
			return t;
		}
	}
	return NULL;
}


TBlock *Net::findTBlockById(const std::string& id)
{
	for (vector<TBlock*>::const_iterator iter = tblocks->begin(); iter != tblocks->end(); iter++)
	{
		TBlock* t = *iter;
		if (*(t->getId()) == id)
		{
			return t;
		}
	}
	return NULL;
}


bool Net::add(Place *p)
{
	Place *pp = findPlaceByName((p->getName()));
	if (pp != NULL)
	{
		std::cout << "A place with name " << p->getName() << " already exists" << std::endl;

		return false;
	}

	places->push_back(p);
	return true;
}

bool Net::add(TBlock *t)
{
	tblocks->push_back(t);
	return true;
}

void Net::print()
{
	std::cout << "Print the net" << std::endl;
	std::cout << "How many places: " << places->size() << std::endl;
	std::cout << "How many transition blocks: " << tblocks->size() << std::endl;

	for (vector<TBlock*>::const_iterator iter = tblocks->begin(); iter != tblocks->end(); iter++)
	{
		TBlock* tb = *iter;

		std::cout << "  tblock " << tb->getName() << std::endl;
		vector<Transition*> *transitions = tb->getTransitions();

		for (vector<Transition*>::const_iterator t_iter = transitions->begin(); t_iter != transitions->end(); t_iter++)
		{
			Transition* t = *t_iter;

			std::cout << "  transition probability " << t->getProbability() << std::endl;
		    std::cout << "    incoming edges " << std::endl;
		    vector<Edge*> *incoming = t->getIncomingEdges();
		    for (vector<Edge*>::const_iterator i_incoming = incoming->begin(); i_incoming != incoming->end(); i_incoming++)
		    {
		    	Edge* e = *i_incoming;
		    	Place* p = e->getPlace();
		    	std::cout << "    place name: " << p->getName() << std::endl;
		    	std::cout << "    place tokens: " << p->getTokensNumber() << std::endl;
		    }

		    std::cout << "    outcoming edges " << std::endl;
		    vector<Edge*> *outcoming = t->getOutcomingEdges();
		    for (vector<Edge*>::const_iterator i_outcoming = outcoming->begin(); i_outcoming != outcoming->end(); i_outcoming++)
		    {
		    	Edge* e = *i_outcoming;
		    	Place* p = e->getPlace();
		    	std::cout << "    place name: " << p->getName() << std::endl;
		    	std::cout << "    place tokens: " << p->getTokensNumber() << std::endl;
		    }
		}
	}
}

void Net::consumeNewlyProduced(){
	for (vector<Place*>::const_iterator i = places->begin(); i!= places->end(); i++)
	{
		Place* p = *i;
		p->consumeNewlyProduced();
	}
}


void Net::setProbabilities(vector<std::string> transitions, vector<int> probabilities)
{
	vector<Transition*> *trs = getTransitions();

	for (int i = 0; i < transitions.size(); i++)
	{
		int found_flag = 0;
		for (vector<Transition*>::const_iterator iter = trs->begin(); iter != trs->end(); iter++)
		{
			Transition *t = *iter;
			if (t->getName() == transitions[i])
			{
				t->setProbability(probabilities[i]);
				found_flag = 1;
			//	std::cout << "transition "<< t->getName() << " probability " << probabilities[i] << std::endl;
			}
		}

		if (found_flag==0)
		{
			std::cout << "transition "<< transitions[i] <<" were not identified!!" << std::endl;
		}
	}
}
